from __future__ import absolute_import

import sys

from types import ModuleType

from .local import Proxy


def _compat_task_decorator(*args, **kwargs):
    from celery import current_app
    kwargs.setdefault("accept_magic_kwargs", True)
    return current_app.task(*args, **kwargs)



def _compat_periodic_task_decorator(*args, **kwargs):
    from celery.task import periodic_task
    kwargs.setdefault("accept_magic_kwargs", True)
    return periodic_task(*args, **kwargs)


modules = {
    "celery": {
        "decorators": {
            "task": _compat_task_decorator,
            "periodic_task": _compat_periodic_task_decorator,
        },
        "log": {
            "get_default_logger": "log.get_default_logger",
            "setup_logger": "log.setup_logger",
            "setup_task_logger": "log.setup_task_logger",
            "get_task_logger": "log.get_task_logger",
            "setup_loggig_subsystem": "log.setup_logging_subsystem",
            "redirect_stdouts_to_logger": "log.redirect_stdouts_to_logger",
        },
        "messaging": {
            "TaskPublisher": "amqp.TaskPublisher",
            "ConsumerSet": "amqp.ConsumerSet",
            "TaskConsumer": "amqp.TaskConsumer",
            "establish_connection": "broker_connection",
            "with_connection": "with_default_connection",
            "get_consumer_set": "amqp.get_task_consumer"
        },
        "registry": {
            "tasks": "tasks",
        },
    },
}


def rgetattr(obj, path):
    return reduce(lambda a, b: getattr(a, b), [obj] + path)


def get_compat(app, pkg, name, bases=(ModuleType, )):
    fqdn = '.'.join([pkg.__name__, name])
    def build_attr(attr):
        if isinstance(attr, basestring):
            return Proxy(rgetattr, (app, attr.split('.')))
        return attr
    attrs = dict((name, build_attr(attr))
                    for name, attr in modules[pkg.__name__][name].iteritems())
    sys.modules[fqdn] = module = type(name, bases, attrs)(fqdn)
    return module
